@@ -36,6 +36,7 @@ except ImportError:
     pymysql = None
 import six
 import sqlalchemy
+import sqlalchemy.engine.url as sqlalchemy_url
 import sqlalchemy.exc
 from sqlalchemy import types
 import sqlalchemy_utils
@@ -217,6 +218,15 @@ class ResourceClassMapper(object):
 class SQLAlchemyIndexer(indexer.IndexerDriver):
     _RESOURCE_TYPE_MANAGER = ResourceClassMapper()
 
+    @classmethod
+    def _create_new_database(cls, url):
+        """Used by testing to create a new database."""
+        purl = sqlalchemy_url.make_url(url)
+        purl.database = purl.database + str(uuid.uuid4()).replace('-', '')
+        new_url = str(purl)
+        sqlalchemy_utils.create_database(new_url)
+        return new_url
+
     def __init__(self, conf):
         conf.set_override("connection", conf.indexer.url, "database")
         self.conf = conf
@@ -783,13 +793,12 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                             #    (pymysql.err.ProgrammingError)
                             #    (1146, "Table \'test.rt_f00\' doesn\'t exist")
                             # In that case, just ignore those resources.
-                            inn_e = e.inner_exception
                             if (not pymysql
                                or not isinstance(
-                                   inn_e, sqlalchemy.exc.ProgrammingError)
+                                   e, sqlalchemy.exc.ProgrammingError)
                                or not isinstance(
-                                   inn_e.orig, pymysql.err.ProgrammingError)
-                               or (inn_e.orig.args[0]
+                                   e.orig, pymysql.err.ProgrammingError)
+                               or (e.orig.args[0]
                                    != pymysql.constants.ER.NO_SUCH_TABLE)):
                                 raise
 
