@@ -751,7 +751,7 @@ class ResourceHistoryController(rest.RestController):
         resource = pecan.request.indexer.get_resource(
             self.resource_type, self.resource_id)
         if not resource:
-            abort(404, "foo")
+            abort(404, indexer.NoSuchResource(self.resource_id))
 
         enforce("get resource", resource)
 
@@ -837,7 +837,10 @@ class ResourceTypesController(rest.RestController):
     def post(self):
         schema = pecan.request.indexer.get_resource_type_schema()
         body = deserialize_and_validate(schema)
-        rt = schema.resource_type_from_dict(**body)
+        try:
+            rt = schema.resource_type_from_dict(**body)
+        except resource_type.InvalidResourceAttributeName as e:
+            abort(400, e)
         enforce("create resource type", body)
         try:
             rt = pecan.request.indexer.create_resource_type(rt)
@@ -1384,7 +1387,8 @@ class StatusController(rest.RestController):
     @pecan.expose('json')
     def get(details=True):
         enforce("get status", {})
-        report = pecan.request.storage.measures_report(details)
+        report = pecan.request.storage.measures_report(
+            strutils.bool_from_string(details))
         report_dict = {"storage": {"summary": report['summary']}}
         if 'details' in report:
             report_dict["storage"]["measures_to_process"] = report['details']
