@@ -13,7 +13,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import errno
 import functools
+import json
 import os
 import uuid
 
@@ -21,8 +23,12 @@ import fixtures
 from oslotest import base
 from oslotest import mockpatch
 import six
+from six.moves.urllib.parse import unquote
 from stevedore import extension
-from swiftclient import exceptions as swexc
+try:
+    from swiftclient import exceptions as swexc
+except ImportError:
+    swexc = None
 from testtools import testcase
 from tooz import coordination
 
@@ -54,13 +60,53 @@ def _skip_decorator(func):
 
 
 class FakeRadosModule(object):
+    class OpCtx(object):
+        def __enter__(self):
+            self.ops = []
+            return self
+
+        def __exit__(self, *args, **kwargs):
+            pass
+
+    WriteOpCtx = ReadOpCtx = OpCtx
+
+    class OmapIterator(object):
+        class OpRetCode(object):
+            def __init__(self):
+                self.ret = 0
+
+            def __eq__(self, other):
+                return self.ret == other
+
+        def __init__(self, start_filter, prefix_filter, number):
+            self.start_filter = start_filter
+            self.prefix_filter = prefix_filter
+            self.number = number
+            self.data = {}
+            self.op_ret = self.OpRetCode()
+
+        def set_data(self, data):
+            if not data:
+                self.op_ret.ret = errno.ENOENT
+            else:
+                self.data = data
+
+        def __iter__(self):
+            # NOTE(sileht): we use only the prefix for now
+            return ((k, v) for k, v in self.data.items()
+                    if k.startswith(self.prefix_filter))
+
+    LIBRADOS_OPERATION_BALANCE_READS = 1
+    LIBRADOS_OPERATION_SKIPRWLOCKS = 16
+
     class ObjectNotFound(Exception):
         pass
 
     class ioctx(object):
-        def __init__(self, kvs, kvs_xattrs):
+        def __init__(self, kvs, kvs_xattrs, kvs_omaps):
             self.kvs = kvs
             self.kvs_xattrs = kvs_xattrs
+            self.kvs_omaps = kvs_omaps
             self.librados = self
             self.io = self
 
@@ -75,21 +121,7 @@ class FakeRadosModule(object):
             if key not in self.kvs:
                 self.kvs[key] = ""
                 self.kvs_xattrs[key] = {}
-
-        def rados_lock_exclusive(self, ctx, name, lock, locker, desc, timeval,
-                                 flags):
-            # Locking a not existing object create an empty one
-            # so, do the same in test
-            key = name.value.decode('ascii')
-            self._ensure_key_exists(key)
-            return 0
-
-        def rados_unlock(self, ctx, name, lock, locker):
-            # Locking a not existing object create an empty one
-            # so, do the same in test
-            key = name.value.decode('ascii')
-            self._ensure_key_exists(key)
-            return 0
+                self.kvs_omaps[key] = {}
 
         @staticmethod
         def close():
@@ -119,11 +151,37 @@ class FakeRadosModule(object):
             else:
                 return self.kvs[key][offset:offset+length]
 
+        def operate_read_op(self, op, key, flag=0):
+            for op in op.ops:
+                op(key)
+
+        def get_omap_vals(self, op, start_filter, prefix_filter, number):
+            oi = FakeRadosModule.OmapIterator(start_filter, prefix_filter,
+                                              number)
+            op.ops.append(lambda oid: oi.set_data(self.kvs_omaps.get(oid)))
+            return oi, oi.op_ret
+
+        def operate_write_op(self, op, key, flags=0):
+            for op in op.ops:
+                op(key)
+
+        def set_omap(self, op,  keys, values):
+            def add(oid):
+                self._ensure_key_exists(oid)
+                omaps = self.kvs_omaps.setdefault(oid, {})
+                omaps.update(dict(zip(keys, values)))
+            op.ops.append(add)
+
+        def remove_omap_keys(self, op,  keys):
+            def rm(oid):
+                for key in keys:
+                    del self.kvs_omaps[oid][key]
+            op.ops.append(rm)
+
         def get_xattrs(self, key):
             if key not in self.kvs:
                 raise FakeRadosModule.ObjectNotFound
-            return iter((k, v) for k, v in
-                        self.kvs_xattrs.get(key, {}).items())
+            return six.iteritems(self.kvs_xattrs.get(key, {}).copy())
 
         def set_xattr(self, key, attr, value):
             self._ensure_key_exists(key)
@@ -141,11 +199,19 @@ class FakeRadosModule(object):
                 raise FakeRadosModule.ObjectNotFound
             del self.kvs[key]
             del self.kvs_xattrs[key]
+            del self.kvs_omaps[key]
+
+        def aio_remove(self, key):
+            self._validate_key(key)
+            self.kvs.pop(key, None)
+            self.kvs_xattrs.pop(key, None)
+            self.kvs_omaps.pop(key, None)
 
     class FakeRados(object):
-        def __init__(self, kvs, kvs_xattrs):
+        def __init__(self, kvs, kvs_xattrs, kvs_omaps):
             self.kvs = kvs
             self.kvs_xattrs = kvs_xattrs
+            self.kvs_omaps = kvs_omaps
 
         @staticmethod
         def connect():
@@ -156,14 +222,17 @@ class FakeRadosModule(object):
             pass
 
         def open_ioctx(self, pool):
-            return FakeRadosModule.ioctx(self.kvs, self.kvs_xattrs)
+            return FakeRadosModule.ioctx(self.kvs, self.kvs_xattrs,
+                                         self.kvs_omaps)
 
     def __init__(self):
         self.kvs = {}
         self.kvs_xattrs = {}
+        self.kvs_omaps = {}
 
     def Rados(self, *args, **kwargs):
-        return FakeRadosModule.FakeRados(self.kvs, self.kvs_xattrs)
+        return FakeRadosModule.FakeRados(self.kvs, self.kvs_xattrs,
+                                         self.kvs_omaps)
 
     @staticmethod
     def run_in_thread(method, args):
@@ -187,7 +256,7 @@ class FakeSwiftClient(object):
         self.kvs[container] = {}
 
     def get_container(self, container, delimiter=None,
-                      path=None, full_listing=False):
+                      path=None, full_listing=False, limit=None):
         try:
             container = self.kvs[container]
         except KeyError:
@@ -196,7 +265,7 @@ class FakeSwiftClient(object):
 
         files = []
         directories = set()
-        for k, v in six.iteritems(container):
+        for k, v in six.iteritems(container.copy()):
             if path and not k.startswith(path):
                 continue
 
@@ -214,12 +283,15 @@ class FakeSwiftClient(object):
 
         if full_listing:
             end = None
+        elif limit:
+            end = limit
         else:
             # In truth, it's 10000, but 1 is enough to make sure our test fails
             # otherwise.
             end = 1
 
-        return {}, (files + list(directories))[:end]
+        return ({'x-container-object-count': len(container.keys())},
+                (files + list(directories))[:end])
 
     def put_object(self, container, key, obj):
         if hasattr(obj, "seek"):
@@ -260,56 +332,27 @@ class FakeSwiftClient(object):
             raise swexc.ClientException("No such container",
                                         http_status=404)
 
+    def post_account(self, headers, query_string=None, data=None):
+        resp = {'Response Status': '200 OK',
+                'Response Body': '',
+                'Number Deleted': 0,
+                'Number Not Found': 0}
+        if query_string == 'bulk-delete' and data:
+            for path in data.splitlines():
+                try:
+                    __, container, obj = (unquote(path.decode('utf8'))
+                                          .split('/', 2))
+                    del self.kvs[container][obj]
+                    resp['Number Deleted'] += 1
+                except KeyError:
+                    resp['Number Not Found'] += 1
+        return {}, json.dumps(resp).encode('utf-8')
+
 
 @six.add_metaclass(SkipNotImplementedMeta)
 class TestCase(base.BaseTestCase):
 
     ARCHIVE_POLICIES = {
-        'low': archive_policy.ArchivePolicy(
-            "low",
-            0,
-            [
-                # 5 minutes resolution for an hour
-                archive_policy.ArchivePolicyItem(
-                    granularity=300, points=12),
-                # 1 hour resolution for a day
-                archive_policy.ArchivePolicyItem(
-                    granularity=3600, points=24),
-                # 1 day resolution for a month
-                archive_policy.ArchivePolicyItem(
-                    granularity=3600 * 24, points=30),
-            ],
-        ),
-        'medium': archive_policy.ArchivePolicy(
-            "medium",
-            0,
-            [
-                # 1 minute resolution for an hour
-                archive_policy.ArchivePolicyItem(
-                    granularity=60, points=60),
-                # 1 hour resolution for a week
-                archive_policy.ArchivePolicyItem(
-                    granularity=3600, points=7 * 24),
-                # 1 day resolution for a year
-                archive_policy.ArchivePolicyItem(
-                    granularity=3600 * 24, points=365),
-            ],
-        ),
-        'high': archive_policy.ArchivePolicy(
-            "high",
-            0,
-            [
-                # 1 second resolution for a day
-                archive_policy.ArchivePolicyItem(
-                    granularity=1, points=3600 * 24),
-                # 1 minute resolution for a month
-                archive_policy.ArchivePolicyItem(
-                    granularity=60, points=60 * 24 * 30),
-                # 1 hour resolution for a year
-                archive_policy.ArchivePolicyItem(
-                    granularity=3600, points=365 * 24),
-            ],
-        ),
         'no_granularity_match': archive_policy.ArchivePolicy(
             "no_granularity_match",
             0,
@@ -334,30 +377,22 @@ class TestCase(base.BaseTestCase):
 
     def setUp(self):
         super(TestCase, self).setUp()
-        self.conf = service.prepare_service([])
+        self.conf = service.prepare_service([],
+                                            default_config_files=[])
         self.conf.set_override('policy_file',
                                self.path_get('etc/gnocchi/policy.json'),
                                group="oslo_policy")
 
-        self.conf.set_override(
-            'url',
-            os.environ.get("GNOCCHI_TEST_INDEXER_URL", "null://"),
-            'indexer')
-
         self.index = indexer.get_driver(self.conf)
         self.index.connect()
 
-        self.conf.set_override('coordination_url',
-                               os.getenv("GNOCCHI_COORDINATION_URL", "ipc://"),
-                               'storage')
-
         # NOTE(jd) So, some driver, at least SQLAlchemy, can't create all
         # their tables in a single transaction even with the
         # checkfirst=True, so what we do here is we force the upgrade code
         # path to be sequential to avoid race conditions as the tests run
         # in parallel.
         self.coord = coordination.get_coordinator(
-            os.getenv("GNOCCHI_COORDINATION_URL", "ipc://"),
+            self.conf.storage.coordination_url,
             str(uuid.uuid4()).encode('ascii'))
 
         self.coord.start()
@@ -368,23 +403,26 @@ class TestCase(base.BaseTestCase):
             # in the Alembic upgrades. We have a test to check that
             # upgrades == create but it misses things such as custom CHECK
             # constraints.
-            self.index.upgrade(nocreate=True)
+            self.index.upgrade(nocreate=True,
+                               create_legacy_resource_types=True)
 
         self.coord.stop()
 
-        self.archive_policies = self.ARCHIVE_POLICIES
+        self.archive_policies = self.ARCHIVE_POLICIES.copy()
+        self.archive_policies.update(archive_policy.DEFAULT_ARCHIVE_POLICIES)
         # Used in gnocchi.gendoc
         if not getattr(self, "skip_archive_policies_creation", False):
-            for name, ap in six.iteritems(self.ARCHIVE_POLICIES):
+            for name, ap in six.iteritems(self.archive_policies):
                 # Create basic archive policies
                 try:
                     self.index.create_archive_policy(ap)
                 except indexer.ArchivePolicyAlreadyExists:
                     pass
 
-        self.useFixture(mockpatch.Patch(
-            'swiftclient.client.Connection',
-            FakeSwiftClient))
+        if swexc:
+            self.useFixture(mockpatch.Patch(
+                'swiftclient.client.Connection',
+                FakeSwiftClient))
 
         self.useFixture(mockpatch.Patch('gnocchi.storage.ceph.rados',
                                         FakeRadosModule()))
@@ -399,22 +437,14 @@ class TestCase(base.BaseTestCase):
             self.conf.set_override('file_basepath',
                                    tempdir.path,
                                    'storage')
-        elif self.conf.storage.driver == 'influxdb':
-            self.conf.set_override('influxdb_block_until_data_ingested', True,
-                                   'storage')
-            self.conf.set_override('influxdb_database', 'test', 'storage')
-            self.conf.set_override('influxdb_password', 'root', 'storage')
-            self.conf.set_override('influxdb_port',
-                                   os.getenv("GNOCCHI_TEST_INFLUXDB_PORT",
-                                             51234), 'storage')
-            # NOTE(ityaptin) Creating unique database for every test may cause
-            # tests failing by timeout, but in may be useful in some cases
-            if os.getenv("GNOCCHI_TEST_INFLUXDB_UNIQUE_DATABASES"):
-                self.conf.set_override("influxdb_database",
-                                       "gnocchi_%s" % uuid.uuid4().hex,
-                                       'storage')
 
         self.storage = storage.get_driver(self.conf)
+        # NOTE(jd) Do not upgrade the storage. We don't really need the storage
+        # upgrade for now, and the code that upgrade from pre-1.3
+        # (TimeSerieArchive) uses a lot of parallel lock, which makes tooz
+        # explodes because MySQL does not support that many connections in real
+        # life.
+        # self.storage.upgrade(self.index)
 
         self.mgr = extension.ExtensionManager('gnocchi.aggregates',
                                               invoke_on_load=True)
