@@ -1,9 +1,8 @@
 # -*- encoding: utf-8 -*-
 #
+# Copyright © 2016 Red Hat, Inc.
 # Copyright © 2014-2015 eNovance
 #
-# Authors: Julien Danjou <julien@danjou.info>
-#
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -23,7 +22,7 @@ import hashlib
 import json
 import uuid
 
-import keystonemiddleware.auth_token
+from keystonemiddleware import fixture as ksm_fixture
 import mock
 from oslo_utils import timeutils
 import six
@@ -41,77 +40,18 @@ from gnocchi import utils
 load_tests = testscenarios.load_tests_apply_scenarios
 
 
-class FakeMemcache(object):
-    VALID_TOKEN_ADMIN = '4562138218392830'
-    ADMIN_TOKEN_HASH = hashlib.sha256(
-        VALID_TOKEN_ADMIN.encode('utf-8')).hexdigest()
+class TestingApp(webtest.TestApp):
+    VALID_TOKEN_ADMIN = str(uuid.uuid4())
     USER_ID_ADMIN = str(uuid.uuid4())
     PROJECT_ID_ADMIN = str(uuid.uuid4())
 
-    VALID_TOKEN = '4562138218392831'
-    TOKEN_HASH = hashlib.sha256(VALID_TOKEN.encode('utf-8')).hexdigest()
+    VALID_TOKEN = str(uuid.uuid4())
     USER_ID = str(uuid.uuid4())
     PROJECT_ID = str(uuid.uuid4())
 
-    VALID_TOKEN_2 = '4562138218392832'
-    TOKEN_2_HASH = hashlib.sha256(VALID_TOKEN_2.encode('utf-8')).hexdigest()
-    # We replace "-" to simulate a middleware that would send UUID in a non
-    # normalized format.
-    USER_ID_2 = str(uuid.uuid4()).replace("-", "")
-    PROJECT_ID_2 = str(uuid.uuid4()).replace("-", "")
-
-    def get(self, key):
-        dt = "2100-01-01T23:59:59"
-        if (key == "tokens/%s" % self.ADMIN_TOKEN_HASH or
-                key == "tokens/%s" % self.VALID_TOKEN_ADMIN):
-            return json.dumps(({'access': {
-                'token': {'id': self.VALID_TOKEN_ADMIN,
-                          'expires': dt},
-                'user': {
-                    'id': self.USER_ID_ADMIN,
-                    'name': 'adminusername',
-                    'tenantId': self.PROJECT_ID_ADMIN,
-                    'tenantName': 'myadmintenant',
-                    'roles': [
-                        {'name': 'admin'},
-                    ]},
-            }}, dt))
-        elif (key == "tokens/%s" % self.TOKEN_HASH or
-                key == "tokens/%s" % self.VALID_TOKEN):
-            return json.dumps(({'access': {
-                'token': {'id': self.VALID_TOKEN,
-                          'expires': dt},
-                'user': {
-                    'id': self.USER_ID,
-                    'name': 'myusername',
-                    'tenantId': self.PROJECT_ID,
-                    'tenantName': 'mytenant',
-                    'roles': [
-                        {'name': 'member'},
-                    ]},
-            }}, dt))
-        elif (key == "tokens/%s" % self.TOKEN_2_HASH or
-              key == "tokens/%s" % self.VALID_TOKEN_2):
-            return json.dumps(({'access': {
-                'token': {'id': self.VALID_TOKEN_2,
-                          'expires': dt},
-                'user': {
-                    'id': self.USER_ID_2,
-                    'name': 'myusername2',
-                    'tenantId': self.PROJECT_ID_2,
-                    'tenantName': 'mytenant2',
-                    'roles': [
-                        {'name': 'member'},
-                    ]},
-            }}, dt))
-
-    @staticmethod
-    def set(key, value, **kwargs):
-        pass
-
-
-class TestingApp(webtest.TestApp):
-    CACHE_NAME = 'fake.cache'
+    VALID_TOKEN_2 = str(uuid.uuid4())
+    USER_ID_2 = str(uuid.uuid4())
+    PROJECT_ID_2 = str(uuid.uuid4())
 
     def __init__(self, *args, **kwargs):
         self.auth = kwargs.pop('auth')
@@ -119,15 +59,14 @@ class TestingApp(webtest.TestApp):
         self.indexer = kwargs.pop('indexer')
         super(TestingApp, self).__init__(*args, **kwargs)
         # Setup Keystone auth_token fake cache
-        self.extra_environ.update({self.CACHE_NAME: FakeMemcache()})
-        self.token = FakeMemcache.VALID_TOKEN
+        self.token = self.VALID_TOKEN
 
     @contextlib.contextmanager
     def use_admin_user(self):
         if not self.auth:
             raise testcase.TestSkipped("No auth enabled")
         old_token = self.token
-        self.token = FakeMemcache.VALID_TOKEN_ADMIN
+        self.token = self.VALID_TOKEN_ADMIN
         try:
             yield
         finally:
@@ -138,7 +77,7 @@ class TestingApp(webtest.TestApp):
         if not self.auth:
             raise testcase.TestSkipped("No auth enabled")
         old_token = self.token
-        self.token = FakeMemcache.VALID_TOKEN_2
+        self.token = self.VALID_TOKEN_2
         try:
             yield
         finally:
@@ -148,7 +87,7 @@ class TestingApp(webtest.TestApp):
         if self.auth:
             req.headers['X-Auth-Token'] = self.token
         response = super(TestingApp, self).do_request(req, *args, **kwargs)
-        self.storage.process_background_tasks(self.indexer, True)
+        self.storage.process_background_tasks(self.indexer, sync=True)
         return response
 
 
@@ -159,57 +98,45 @@ class RestTest(tests_base.TestCase, testscenarios.TestWithScenarios):
         ('keystone', dict(auth=True)),
     ]
 
-    @classmethod
-    def app_factory(cls, global_config, **local_conf):
-        return app.setup_app(cls.pecan_config, cls.conf)
-
-    @classmethod
-    def keystone_authtoken_filter_factory(cls, global_conf, **local_conf):
-        def auth_filter(app):
-            return keystonemiddleware.auth_token.AuthProtocol(app, {
-                "oslo_config_project": "gnocchi",
-                "oslo_config_config": cls.conf,
-            })
-        return auth_filter
-
     def setUp(self):
         super(RestTest, self).setUp()
         self.conf.set_override('paste_config',
-                               self.path_get('gnocchi/tests/api-paste.ini'),
+                               self.path_get('etc/gnocchi/api-paste.ini'),
                                group="api")
-        pecan_config = {}
-        pecan_config.update(app.PECAN_CONFIG)
-        pecan_config['indexer'] = self.index
-        pecan_config['storage'] = self.storage
-        pecan_config['not_implemented_middleware'] = False
 
-        self.conf.set_override("cache", TestingApp.CACHE_NAME,
-                               group='keystone_authtoken')
-        # TODO(jd) Override these options with values. They are not used, but
-        # if they are None (their defaults), the keystone authtoken middleware
-        # prints a warning… :( When the bug is fixed we can remove that!
-        # See https://bugs.launchpad.net/keystonemiddleware/+bug/1429179
-        self.conf.set_override("identity_uri", "foobar",
-                               group="keystone_authtoken")
-        self.conf.set_override("auth_uri", "foobar",
-                               group="keystone_authtoken")
-        self.conf.set_override("delay_auth_decision",
-                               not self.auth,
-                               group="keystone_authtoken")
+        self.auth_token_fixture = self.useFixture(
+            ksm_fixture.AuthTokenFixture())
+        self.auth_token_fixture.add_token_data(
+            is_v2=True,
+            token_id=TestingApp.VALID_TOKEN_ADMIN,
+            user_id=TestingApp.USER_ID_ADMIN,
+            user_name='adminusername',
+            project_id=TestingApp.PROJECT_ID_ADMIN,
+            role_list=['admin'])
+        self.auth_token_fixture.add_token_data(
+            is_v2=True,
+            token_id=TestingApp.VALID_TOKEN,
+            user_id=TestingApp.USER_ID,
+            user_name='myusername',
+            project_id=TestingApp.PROJECT_ID,
+            role_list=["member"])
+        self.auth_token_fixture.add_token_data(
+            is_v2=True,
+            token_id=TestingApp.VALID_TOKEN_2,
+            user_id=TestingApp.USER_ID_2,
+            user_name='myusername2',
+            project_id=TestingApp.PROJECT_ID_2,
+            role_list=["member"])
 
-        RestTest.pecan_config = pecan_config
-        RestTest.conf = self.conf
-
-        # TODO(chdent) Linting is turned off until a
-        # keystonemiddleware bug is resolved.
-        # See: https://bugs.launchpad.net/keystonemiddleware/+bug/1466499
-        self.app = TestingApp(app.load_app(self.conf,
-                                           appname="testing+auth"
-                                           if self.auth else "testing"),
+        self.app = TestingApp(app.load_app(conf=self.conf,
+                                           appname="gnocchi+auth"
+                                           if self.auth else "gnocchi+noauth",
+                                           indexer=self.index,
+                                           storage=self.storage,
+                                           not_implemented_middleware=False),
                               storage=self.storage,
                               indexer=self.index,
-                              auth=self.auth,
-                              lint=False)
+                              auth=self.auth)
 
     def test_deserialize_force_json(self):
         with self.app.use_admin_user():
@@ -239,8 +166,9 @@ class RestTest(tests_base.TestCase, testscenarios.TestWithScenarios):
         status = json.loads(r.text)
         # We are sure this is empty because we call process_measures() each
         # time we do a REST request in this TestingApp.
-        self.assertEqual({},
-                         status['storage']['measures_to_process'])
+        self.assertEqual({}, status['storage']['measures_to_process'])
+        self.assertIs(type(status['storage']['summary']['metrics']), int)
+        self.assertIs(type(status['storage']['summary']['measures']), int)
 
     @staticmethod
     def runTest():
@@ -325,6 +253,21 @@ class ArchivePolicyTest(RestTest):
 
 class MetricTest(RestTest):
 
+    def test_get_metric_with_another_user_linked_resource(self):
+        result = self.app.post_json(
+            "/v1/resource/generic",
+            params={
+                "id": str(uuid.uuid4()),
+                "started_at": "2014-01-01 02:02:02",
+                "user_id": TestingApp.USER_ID_2,
+                "project_id": TestingApp.PROJECT_ID_2,
+                "metrics": {"foobar": {"archive_policy_name": "low"}},
+            })
+        resource = json.loads(result.text)
+        metric_id = resource["metrics"]["foobar"]
+        with self.app.use_another_user():
+            self.app.get("/v1/metric/%s" % metric_id)
+
     def test_get_metric_with_another_user(self):
         result = self.app.post_json("/v1/metric",
                                     params={"archive_policy_name": "medium"},
@@ -379,11 +322,6 @@ class MetricTest(RestTest):
                 status=403)
 
     def test_add_measures_back_window(self):
-        if self.conf.storage.driver == 'influxdb':
-            # FIXME(sileht): Won't pass with influxdb because it doesn't
-            # check archive policy
-            raise testcase.TestSkipped("InfluxDB issue")
-
         ap_name = str(uuid.uuid4())
         with self.app.use_admin_user():
             self.app.post_json(
@@ -457,6 +395,7 @@ class MetricTest(RestTest):
         result = json.loads(ret.text)
         now = utils.utcnow()
         self.assertEqual([
+            ['2014-01-01T10:00:00+00:00', 3600.0, 1234.2],
             [(now
               - datetime.timedelta(
                   seconds=now.second,
@@ -811,6 +750,41 @@ class ResourceTest(RestTest):
                 "ended_at": "2014-01-03T02:02:02+00:00",
             },
             resource_type='image')),
+        ('host', dict(
+            attributes={
+                "started_at": "2014-01-03T02:02:02+00:00",
+                "user_id": str(uuid.uuid4()),
+                "project_id": str(uuid.uuid4()),
+                "host_name": "test-host",
+            },
+            patchable_attributes={
+                "ended_at": "2014-01-03T02:02:02+00:00",
+            },
+            resource_type='host')),
+        ('host_disk', dict(
+            attributes={
+                "started_at": "2014-01-03T02:02:02+00:00",
+                "user_id": str(uuid.uuid4()),
+                "project_id": str(uuid.uuid4()),
+                "host_name": "test-host",
+                "device_name": "test-device"
+            },
+            patchable_attributes={
+                "ended_at": "2014-01-03T02:02:02+00:00",
+            },
+            resource_type='host_disk')),
+        ('host_network_interface', dict(
+            attributes={
+                "started_at": "2014-01-03T02:02:02+00:00",
+                "user_id": str(uuid.uuid4()),
+                "project_id": str(uuid.uuid4()),
+                "host_name": "test-host",
+                "device_name": "test-device"
+            },
+            patchable_attributes={
+                "ended_at": "2014-01-03T02:02:02+00:00",
+            },
+            resource_type='host_network_interface')),
     ]
 
     @classmethod
@@ -826,9 +800,11 @@ class ResourceTest(RestTest):
         # Set an id in the attribute
         self.attributes['id'] = str(uuid.uuid4())
         self.resource = self.attributes.copy()
+        # Set original_resource_id
+        self.resource['original_resource_id'] = self.resource['id']
         if self.auth:
-            self.resource['created_by_user_id'] = FakeMemcache.USER_ID
-            self.resource['created_by_project_id'] = FakeMemcache.PROJECT_ID
+            self.resource['created_by_user_id'] = TestingApp.USER_ID
+            self.resource['created_by_project_id'] = TestingApp.PROJECT_ID
         else:
             self.resource['created_by_user_id'] = None
             self.resource['created_by_project_id'] = None
@@ -1496,6 +1472,20 @@ class ResourceTest(RestTest):
         self.assertGreaterEqual(len(resources), 1)
         self.assertEqual(result, resources[0])
 
+    def test_search_resource_by_original_resource_id(self):
+        result = self.app.post_json(
+            "/v1/resource/" + self.resource_type,
+            params=self.attributes)
+        created_resource = json.loads(result.text)
+        original_id = created_resource['original_resource_id']
+        result = self.app.post_json(
+            "/v1/search/resource/" + self.resource_type,
+            params={"eq": {"original_resource_id": original_id}},
+            status=200)
+        resources = json.loads(result.text)
+        self.assertGreaterEqual(len(resources), 1)
+        self.assertEqual(created_resource, resources[0])
+
     def test_search_resources_by_user(self):
         u1 = str(uuid.uuid4())
         self.attributes['user_id'] = u1
@@ -1516,6 +1506,49 @@ class ResourceTest(RestTest):
         self.assertGreaterEqual(len(resources), 1)
         self.assertEqual(created_resource, resources[0])
 
+    def test_search_resources_with_another_project_id(self):
+        u1 = str(uuid.uuid4())
+        result = self.app.post_json(
+            "/v1/resource/generic",
+            params={
+                "id": str(uuid.uuid4()),
+                "started_at": "2014-01-01 02:02:02",
+                "user_id": u1,
+                "project_id": TestingApp.PROJECT_ID_2,
+            })
+        g = json.loads(result.text)
+
+        with self.app.use_another_user():
+            result = self.app.post_json(
+                "/v1/resource/generic",
+                params={
+                    "id": str(uuid.uuid4()),
+                    "started_at": "2014-01-01 03:03:03",
+                    "user_id": u1,
+                    "project_id": str(uuid.uuid4()),
+                })
+            j = json.loads(result.text)
+            g_found = False
+            j_found = False
+
+            result = self.app.post_json(
+                "/v1/search/resource/generic",
+                params={"=": {"user_id": u1}},
+                status=200)
+            resources = json.loads(result.text)
+            self.assertGreaterEqual(len(resources), 2)
+            for r in resources:
+                if r['id'] == str(g['id']):
+                    self.assertEqual(g, r)
+                    g_found = True
+                elif r['id'] == str(j['id']):
+                    self.assertEqual(j, r)
+                    j_found = True
+                if g_found and j_found:
+                    break
+            else:
+                self.fail("Some resources were not found")
+
     def test_search_resources_by_unknown_field(self):
         result = self.app.post_json(
             "/v1/search/resource/" + self.resource_type,
@@ -1640,6 +1673,47 @@ class ResourceTest(RestTest):
         else:
             self.fail("Some resources were not found")
 
+    def test_list_resources_with_another_project_id(self):
+        result = self.app.post_json(
+            "/v1/resource/generic",
+            params={
+                "id": str(uuid.uuid4()),
+                "started_at": "2014-01-01 02:02:02",
+                "user_id": TestingApp.USER_ID_2,
+                "project_id": TestingApp.PROJECT_ID_2,
+            })
+        g = json.loads(result.text)
+
+        with self.app.use_another_user():
+            result = self.app.post_json(
+                "/v1/resource/generic",
+                params={
+                    "id": str(uuid.uuid4()),
+                    "started_at": "2014-01-01 03:03:03",
+                    "user_id": str(uuid.uuid4()),
+                    "project_id": str(uuid.uuid4()),
+                })
+            j = json.loads(result.text)
+
+            g_found = False
+            j_found = False
+
+            result = self.app.get("/v1/resource/generic")
+            self.assertEqual(200, result.status_code)
+            resources = json.loads(result.text)
+            self.assertGreaterEqual(len(resources), 2)
+            for r in resources:
+                if r['id'] == str(g['id']):
+                    self.assertEqual(g, r)
+                    g_found = True
+                elif r['id'] == str(j['id']):
+                    self.assertEqual(j, r)
+                    j_found = True
+                if g_found and j_found:
+                    break
+            else:
+                self.fail("Some resources were not found")
+
     def test_list_resources_with_details(self):
         self._do_test_list_resources_with_detail(
             lambda: self.app.get("/v1/resource/generic?details=true"))
@@ -1918,8 +1992,8 @@ class GenericResourceTest(RestTest):
             params={
                 "id": resource_id,
                 "started_at": "2014-01-01 02:02:02",
-                "user_id": FakeMemcache.USER_ID_2,
-                "project_id": FakeMemcache.PROJECT_ID_2,
+                "user_id": TestingApp.USER_ID_2,
+                "project_id": TestingApp.PROJECT_ID_2,
                 "metrics": {"foobar": {"archive_policy_name": "low"}},
             })
 
